l("JS N plain");
ln("JS N one", "JS N many", 4);
lp("JS_ctx", "JS N ctx plain");
lnp("JS_ctx", "JS N ctx one", "JS N ctx many", 4);
ld("JS_dom", "JS N dom plain");
ldn("JS_dom", "JS N dom one", "JS N dom many", 4);
ldp("JS_dom", "JS_ctx", "JS N dom ctx plain");
ldnp("JS_dom", "JS_ctx", "JS N dom ctx one", "JS N dom ctx many", 4);
