l("N plain");
ln("N one", "N many", 4);
lp("N_ctx", "N ctx plain");
lnp("N_ctx", "N ctx one", "N ctx many", 4);
ld("N_dom", "N dom plain");
ldn("N_dom", "N dom one", "N dom many", 4);
ldp("N_dom", "N_ctx", "N dom ctx plain");
ldnp("N_dom", "N_ctx", "N dom ctx one", "N dom ctx many", 4);
