N_("JS N plain");
Nn_("JS N one", "JS N many");
Np_("JS_ctx", "JS N ctx plain");
Nnp_("JS_ctx", "JS N ctx one", "JS N ctx many");
Nd_("JS_dom", "JS N dom plain");
Ndn_("JS_dom", "JS N dom one", "JS N dom many");
Ndp_("JS_dom", "JS_ctx", "JS N dom ctx plain");
Ndnp_("JS_dom", "JS_ctx", "JS N dom ctx one", "JS N dom ctx many");
