N_("N plain");
Nn_("N one", "N many");
Np_("N_ctx", "N ctx plain");
Nnp_("N_ctx", "N ctx one", "N ctx many");
Nd_("N_dom", "N dom plain");
Ndn_("N_dom", "N dom one", "N dom many");
Ndp_("N_dom", "N_ctx", "N dom ctx plain");
Ndnp_("N_dom", "N_ctx", "N dom ctx one", "N dom ctx many");
